--[[
Queue Engine
]]--

require "draw"
require "gameinit"
require "player"
require "map"
require "update"
require "resources"
require "door"
require "enemy"
require "collisionline"

bit = require "bit"
ATL = require "AdvTiledLoader"
gamera = require "gamera"
anim8 = require "anim8/anim8"

lg = love.graphics

WALK = 300
RUN = 500

JUMPBOOST = 150
WALLJUMP = 600
WALLJUMP_DECREMENT = 40
STICKSLOWDOWN = 1300

WALLJUMP_VERT = -900
JUMP_VERT = -800 --OR -1035

FLY_SPEED = 1000

function love.load ()
  mpath = "maps/"
  ipath = "img/"
  
  lg.setBackgroundColor(220,220,255)
  levelNum = 1
  
  ATL.Loader.path = mpath
  
  loadMap(1)
  
  map:setDrawRange(0, 0, map.width * map.tileWidth, map.height * map.tileHeight)
	
  
  camera = gamera.new(0, 0, (map.width * map.tileWidth), (map.height * map.tileHeight))
  camera:setWindow(0,0,800,450)
  
  gameInit()
  
  p.x = map.startx
  p.y = map.starty
end

function love.update (dt)
  if dt > 0.05 then
		dt = 0.05
	end

  pCtrl(dt)
  pDeathUpdate(dt)
  pAnimationUpdate(dt)
  
	for i,v in ipairs(map.enemies) do
      v:update(dt)
      --v:updateMoves(dt, world.gravity)
  end
  
	camera:setScale(1.5)
	camera:setPosition(p.x, p.y)
end

function love.draw ()
    
  camera:draw(function(l,t,w,h)
		
    for i,v in ipairs(map.doors) do
      v:draw()
    end
    
    for i,v in ipairs(map.enemies) do
      v:draw()
    end
    
	if not (isDead) then
      if (p.state == "stand") then
        pstand:draw(pimage, (p.x-16), (p.y-16))
      elseif (p.state == "right") and (p.speed_status == "run") then
        prunr:draw(pimage, (p.x-16), (p.y-16))
      elseif (p.state == "left") and (p.speed_status == "run") then
        prunl:draw(pimage, (p.x-16), (p.y-16))
      elseif (p.state == "right") and (p.speed_status == "walk") then
        pwalkr:draw(pimage, (p.x-16), (p.y-16))
      elseif (p.state == "left") and (p.speed_status == "walk") then
        pwalkl:draw(pimage, (p.x-16), (p.y-16))
      elseif (p.state == "jump") and (p.x_vel >= 0) then
        pjumpr:draw(pimage, (p.x-16), (p.y-16))
      elseif (p.state == "jump") and (p.x_vel < 0) then
        pjumpl:draw(pimage, (p.x-16), (p.y-16))
      elseif (p.state == "fall") and (p.x_vel >= 0) then
        pfallr:draw(pimage, (p.x-16), (p.y-16))
      elseif (p.state == "fall") and (p.x_vel < 0) then
        pfalll:draw(pimage, (p.x-16), (p.y-16))
      end
	end
		
		map:draw()
		
  end)
  debugText()
end

function love.keyreleased(key)
  if key == "escape" then
        love.event.push("quit")  -- actually causes the app to quit
  end
	if (key == "left") or (key == "right") then
		p.x_vel = 0
	end
	if (key == "lshift") or (key == "rshift") then
		p.slowMove = false
	end
  if (key == "x") then
    for i,v in ipairs(map.doors) do
      if v:collidePlayer(p) then
        v.isOn = true
        v:openDoor(p)
      end
    end
    -- print ("x is pressed")
  end
end

function math.clamp(x, min, max)
  return x < min and min or (x > max and max or x)
end

function math.fastabs(num)
  firstshift = bit.brshift(num, 31)
  value = (bit.bxor(num, firstshift)) - firstshift
  return value
end