Player = 	{}

function Player:new()
  local object = {
      x = 256,
      y = 256,
      x_vel = 0,
      y_vel = 0,
      jump_vel = JUMP_VERT,
      wall_jump_vel = WALLJUMP_VERT,
      speed = WALK,
      speed_status = "walk",
      flySpeed = FLY_SPEED,
      y_stick = 0,
      side_stick = 0,
      horizontal_jump_vel = 0,
      state = "",
      h = 32,
      w = 16,
			standing = false,
      bJumpLetGo = true,
      lives = 3,
    }
    
    setmetatable(object, { __index = Player })
    return object
end

function Player:jump()
		if self.standing and self.bJumpLetGo then
			self.y_vel = self.jump_vel
			self.speed = self.speed + JUMPBOOST
			self.standing = false
			self.bJumpLetGo = false
		elseif not(self.side_stick == 0) and self.bJumpLetGo then
			self.y_vel = self.wall_jump_vel
			if self.speed_status == "run" then
				self.speed = RUN + JUMPBOOST
			else
				self.speed = WALK + JUMPBOOST
			end
			if self.side_stick == -1 then
				self.horizontal_jump_vel = WALLJUMP
			elseif self.side_stick == 1 then
				self.horizontal_jump_vel = -1*WALLJUMP
			end
			self.standing = false
			self.bJumpLetGo = false
		end
end

function Player:jumpLetGo()
	if self.standing or not(self.side_stick == 0) then
		self.bJumpLetGo = true
	end
end

	function Player:right()
		if self.y_stick == 0 then
			self.x_vel = self.speed
		end
	end
	
	function Player:left()
		if self.y_stick == 0 then
			self.x_vel = -1*self.speed
		end
	end
	
	function Player:stop()
		self.x_vel = 0
	end
	
	function Player:walk()
		if self.standing then
			self.speed = WALK
		end
		self.speed_status = "walk"
	end
	
	function Player:run()
		if self.standing then
			self.speed = RUN
		end
		self.speed_status = "run"
	end
	
	function Player:collide(event)
		if event == "floor" then
			self.y_vel = 0
			self.standing = true
		end
		if event == "ceiling" then
			self.y_vel = 0
		end
	end

function Player:update(dt)
  local halfX = self.w / 2
  local halfY = self.h / 2
  
  self.y_vel = self.y_vel + ((world.gravity - self.y_stick) * dt)
  
  if self.standing then
    self.horizontal_jump_vel = 0
  elseif self.horizontal_jump_vel > 0 then
    if self.x_vel > 0 then
      self.horizontal_jump_vel = 0
    else
      self.horizontal_jump_vel = self.horizontal_jump_vel - WALLJUMP_DECREMENT
    end
  elseif self.horizontal_jump_vel < 0 then
    if self.x_vel < 0 then
      self.horizontal_jump_vel = 0
    else
      self.horizontal_jump_vel = self.horizontal_jump_vel + WALLJUMP_DECREMENT
    end
  end
  
  self.x_vel = math.clamp(self.x_vel, -self.speed, self.speed)
  self.y_vel = math.clamp(self.y_vel, -self.flySpeed, self.flySpeed)
  
  local nextY = self.y + (self.y_vel*dt)
  local nextX
  if self.horizontal_jump_vel == 0 then
    nextX = self.x + (self.x_vel*dt)
  else
    nextX = self.x + (self.horizontal_jump_vel*dt)
  end
  
  if self.y_vel < 0 then
    if not (self:isColliding(map,self.x-halfX+1, nextY - halfY))
      and not (self:isColliding(map,self.x + halfX - 1, nextY - halfY)) then
      self.y = nextY
      self.standing = false
    else
      self.y = nextY + map.tileHeight - ((nextY - halfY) % map.tileHeight)
      self:collide("ceiling")
    end
    self.y_stick = 0
  elseif self.y_vel > 0 then
    if not (self:isColliding(map, self.x-halfX+1, nextY + halfY))
      and not (self:isColliding(map, self.x+halfX-1,nextY+halfY)) then
      self.y = nextY
      self.standing = false
      if (self:isColliding(map, self.x + halfX, self.y)) then
        self.side_stick = 1
        self.y_stick = STICKSLOWDOWN
      elseif (self:isColliding(map, self.x - halfX-1, self.y)) then
        self.side_stick = -1
        self.y_stick = STICKSLOWDOWN
      else
        self.side_stick = 0
        self.y_stick = 0
      end
    else
      self.y = nextY - ((nextY + halfY) % map.tileHeight)
      self:collide("floor")
      self.side_stick = 0
      self.y_stick = 0
    end
  end
  
  if self.x_vel > 0 or nextX > self.x then
    if not (self:isColliding(map, nextX + halfX, self.y))
      and not (self:isColliding(map, nextX + halfX, self.y + halfY - 1)) then
      self.x = nextX
    else
      self.x = nextX - ((nextX + halfX) % map.tileWidth)
    end
  elseif self.x_vel < 0 or nextX < self.x then
    if not (self:isColliding(map, nextX - halfX, self.y))
      and not (self:isColliding(map, nextX - halfX, self.y + halfY - 1)) then
      self.x = nextX
    else
      self.x = nextX + map.tileWidth - ((nextX - halfX) % map.tileWidth)
    end
  end
  
  self.state = self:getState()
end

function Player:isColliding(map, x, y)
  local layer = map.tl["solid"]
  local tileX, tileY = math.floor(x / map.tileWidth), math.floor(y / map.tileHeight)
  local tile = layer.tileData(tileX,tileY)
  return not(tile == nil)
end

function Player:getState()
	local tempState = ""
	if self.standing then
		if self.x_vel > 0 then
			tempState = "right"
		elseif self.x_vel < 0 then
			tempState = "left"
		else
			tempState = "stand"
		end
	end
	if self.y_vel < 0 then
		tempState = "jump"
	elseif self.y_vel > 0 then
		tempState = "fall"
	end
  if (self.y > map.height * map.tileHeight - self.h) then
    tempState = "death"
  end
	return tempState
end